import { world } from "@minecraft/server";
import { VERTICAL_SLABS } from "../../library/data/blockList";
import { verticalSlabStates } from "../../library/data/blockState";
import { copperTransformationType } from "../../library/enum/system";
import { Util } from "../../library/utilities/engineUtil";
import { BlockUtil } from "../../library/utilities/blockUtil";

export class VerticalSlab {
    verticalSlab;
    #copperData;
    #copperStage;

    constructor(block) {
        this.location = block.location;
        if (Object.keys(VERTICAL_SLABS).includes(block.typeId)) {
            this.verticalSlab = block;
            this.dimension = block.dimension.id;
            this.id = block.typeId;
            this.data = VERTICAL_SLABS[this.id];
            this.#copperData = this.data?.copperData;
            this.#copperStage = this.#copperData?.stage;
        };
    }

    get isVerticalSlab() {
        return VerticalSlab.checkIsVerticalSlab(this.verticalSlab?.typeId);
    }

    static checkIsVerticalSlab(id) {
        return Object.keys(VERTICAL_SLABS).includes(id);
    }

    #findVerticalSlab(callback) {
        for (const [key, value] of Object.entries(VERTICAL_SLABS)) {
            if (callback(value)) return key;
        };
    }

    #loadStructure(name, dim, {x, y, z}, dir = "north") {
        world.getDimension(dim).runCommand(`structure load ${name.slice(5)}:${dir} ${x} ${y} ${z}`);
    }

    #wax() {
        if (!this.#copperData.isWaxed) {
            this.transVerticalSlabType(this.#findVerticalSlab(data => data?.copperData?.stage == this.#copperStage && data?.copperData?.isWaxed == true));
        };
    }

    #scrape() {
        if (this.#copperData.isWaxed) { // Dewax
            this.transVerticalSlabType(this.#findVerticalSlab(data => data?.copperData?.stage == this.#copperStage && data?.copperData?.isWaxed == false));
        } else if (this.#copperStage >= 1) { // Deoxidize
            this.transVerticalSlabType(this.#findVerticalSlab(data => data?.copperData?.stage == this.#copperStage - 1 && data?.copperData?.isWaxed == false));
        };
    }

    #oxidize() {
        if (this.#copperStage < 3) {
            this.transVerticalSlabType(this.#findVerticalSlab(data => data?.copperData?.stage == this.#copperStage + 1 && data?.copperData?.isWaxed == false));
        };
    }

    getDirection() {
        return this.verticalSlab.permutation.getState(Object.keys(verticalSlabStates)[0]);
    }

    setStates(states = {}) {
        BlockUtil.setStates(this.verticalSlab, states);
    }

    transVerticalSlabType(newVerticalSlabId) {
        if (this.isVerticalSlab) {
            if (this.verticalSlab.isWaterlogged) {
                this.#loadStructure(newVerticalSlabId, this.dimension, this.location, this.getDirection());
            } else {
                BlockUtil.setBlock(this.dimension, this.location, newVerticalSlabId, this.verticalSlab.permutation.getAllStates());
            };
        };
    }

    merge() {
        if (this.isVerticalSlab) {
            const doubleVerticalSlab = this.id.slice(0, 5) + "double_" + this.id.slice(5);
            let state = {};
            if (VERTICAL_SLABS[this.id].hasDirection) {
                state = {
                    "slab:double_vertical_slab_axis": Util.axis(this.getDirection())
                };
            };
            BlockUtil.setBlock(this.dimension, this.location, doubleVerticalSlab, state);
            return true;
        };
        return false;
    }

    waterlog() {
        if (this.isVerticalSlab) {
            this.#loadStructure(this.id, this.dimension, this.location, this.getDirection());
            return true;
        };
        return false;
    }

    transCopper(transType) {
        if (this.isVerticalSlab) {
            if (this.#copperData != undefined) {
                switch(transType) {
                    case copperTransformationType.WAX: this.#wax();
                        break;
                    case copperTransformationType.SCRAPE: this.#scrape();
                        break;
                    case copperTransformationType.OXIDIZE: this.#oxidize();
                        break;
                    default: this.#oxidize();
                };
            };
        };
    }
}