import { system, world } from "@minecraft/server";
import { lowerCaseFirst } from "../../library/utilities/engineUtil";
import { verticalSlabStates } from "../../library/data/blockState";
import { Util } from "../../library/utilities/engineUtil";
import { VerticalSlab } from "../block/verticalSlab";
import { VerticalSlabSet } from "../block/verticalSlabSet";
import { PlayerData } from "../entity/player";

export class VerticalSlabPlacement {
    #delay = false;
    #canInteract = false;
    #canPlace = false;
    #hasWater = new Map();

    constructor() {
        this.verticalSlabPlacement;
        this.verticalSlabDir = "north";
        this.item;
        this.player;
        this.targetToMerge;
    }

    subscribe() {
        world.beforeEvents.itemUseOn.subscribe(data => {
            this.verticalSlabPlacement = new VerticalSlabSet(data);
            this.player = new PlayerData(data.source);
            this.item = this.verticalSlabPlacement.itemId;
            this.#beforeUseOn(data);
        });
        world.afterEvents.itemStartUseOn.subscribe(() => this.#startUseOn());
        world.afterEvents.itemUse.subscribe(() => this.#afterUse());
        world.afterEvents.itemStopUseOn.subscribe(() => this.#stopUseOn());
        world.afterEvents.playerPlaceBlock.subscribe(data => this.#placeBlock(data));
    }

    #beforeUseOn(data) {
        if (this.#delay) {
            data.cancel = true;
        };
        this.isVerticalSlab = false;
        this.#canPlace = false;
        this.targetToMerge = undefined;
        const selectedBlock = data.block;
        const aimBlock = this.verticalSlabPlacement.aimBlock;
        if (VerticalSlab.checkIsVerticalSlab(this.item)) {
            this.isVerticalSlab = true;

            /*
            * Whether the vertical slab is waterlogged after placing it. 
            */
            const isStillWater = aimBlock.typeId == "minecraft:water" && aimBlock.permutation.getState("liquid_depth") == 0;
            this.#hasWater.set(JSON.stringify(selectedBlock.location), selectedBlock.isWaterlogged);
            this.#hasWater.set(JSON.stringify(aimBlock.location), isStillWater ? true : aimBlock.isWaterlogged);

            /*
            * Place the vertical slab and set the states.
            */
            this.verticalSlabDir = lowerCaseFirst(this.verticalSlabPlacement.evaluateDirection());

            /*
            * Merge the vertical slab.
            */
            this.targetToMerge = this.verticalSlabPlacement.evaluateMergeCondition();
            let isSelectingVerticalSlab = false;
            if (JSON.stringify(this.targetToMerge?.location) == JSON.stringify(this.verticalSlabPlacement.location)) {
                isSelectingVerticalSlab = true;
                data.cancel = true;
            };
            system.runTimeout(() => {
                if ((isSelectingVerticalSlab ? true : !this.#canInteract) && this.targetToMerge != undefined) {
                    const mergeSuccess = this.targetToMerge.merge();
                    if(mergeSuccess) {
                        world.playSound(this.targetToMerge.data.sound, Util.transToWorldLoc(this.targetToMerge.location));
                        this.player.decrementStack();
                    };
                    this.delay = true;
                    system.runTimeout(() => this.delay = false, 3);
                }
            }, 1)
        }
    }

    #startUseOn() {
        this.#canInteract = true;
    }

    #afterUse() {
        if (!this.#delay && !this.#canPlace && this.isVerticalSlab && this.targetToMerge != undefined) {
            const mergeSuccess = this.targetToMerge.merge();
            if(mergeSuccess) {
                world.playSound(this.targetToMerge.data.sound, Util.transToWorldLoc(this.targetToMerge.location));
                this.player.decrementStack();
            };
            this.#delay = true;
            system.runTimeout(() => {
                this.#delay = false;
            }, 3);
        }
    }

    #stopUseOn() {
        system.runTimeout(() => {
            this.#canInteract = false;
        }, 1);
    }

    #placeBlock(data) {
        if (this.isVerticalSlab) {
            const block = new VerticalSlab(data.block);
            if (this.#hasWater.get(JSON.stringify(block.location))) {
                block.waterlog();
            };
            verticalSlabStates["minecraft:cardinal_direction"] = this.verticalSlabDir;
            block.setStates(verticalSlabStates);
            system.runTimeout(() => {
                this.#canInteract = false;
            }, 1);
            this.#canPlace = true;
        }
    }
}

export const onSetVerticalSlab = new VerticalSlabPlacement();